



//Triangle Definition: Composed by 3 Vertexs Coords 
struct Triangle{
	int i,j,k;	//Fist, Second and Third  Vertex Indexs 
};



//Error Handler Function. Shows Error Message And Ends Application
void DieWithError(char*ErrorMessage,...){
	char Buffer[50];			//String Buffer
	char FinalMessage[100];			//Final Message Buffer
	//char *StartPtr=ErrorMessage;
	int i=0,len;

	va_list Args;
	va_start (Args,ErrorMessage);

	while(*ErrorMessage){	
		if(*ErrorMessage=='%'){
			len=sprintf(Buffer,"%s", va_arg(Args, char *));
			strcat(&FinalMessage[i],Buffer);
			i=i+len;
			ErrorMessage++;					//Jump 's' char
		}	
		else {
			FinalMessage[i++]=*ErrorMessage;
			//printf("%c %s\n",*ErrorMessage,FinalMessage);
		}
		ErrorMessage++;
		
	}

	FinalMessage[i]='\0';
	va_end (Args);

	//printf("Msg=%s FinalMessage=%s\n",StartPtr,FinalMessage);
	perror(FinalMessage);						//Display Error Message
	exit(1);							//End Application

}





//Perform Delaunay Triangulation
//Receives:	(xVector,yVector) -> each (1xN) Containing Key Points to Triangulate
//Returns:	Triangles -> List of Delaunay Triangles Vertexs
void Delaunay(CvMat*xVector,CvMat*yVector,vector<struct Triangle>&Triangles){
	int n,i,j,k,m;
	bool flag;
	int xn,yn,zn;	
	int*x,*y,*z;
	struct Triangle DelaunayTriangle;	//Triangle Definition

	//Test Input Data
	if(xVector==NULL||yVector==NULL) DieWithError("Delaunay: Invalid Inputs");
	else if(xVector->cols!=yVector->cols || xVector->rows!=yVector->rows) DieWithError("Delaunay: Inputs Size Doesn't Match");

	//Clear all Previuos Triangles form Vector
	Triangles.clear();

	//Number of Points
	n=xVector->cols;

	//Memory Allocation
	x=(int*)malloc(n*sizeof(int));
	y=(int*)malloc(n*sizeof(int));
	z=(int*)malloc(n*sizeof(int));

	//Copy Input Points
	for(i=0;i<n;i++){
		x[i]=cvRound(cvM32f(xVector,0,i));
		y[i]=cvRound(cvM32f(yVector,0,i)); 

		z[i] = x[i]*x[i] + y[i]*y[i];	//Compute Hiperbolóid Projection: z = x^2 + y^2.
	}
	
	//for(i=0;i<n;i++) cvDrawPlus(Image,x[i],y[i],CV_RGB(255,0,0),3,1);	//Plot Points


	//Joseph O'Rourke O(n4) Delaunay Triangulation From [Computational Geometry in C] book
	//For Each Triple (i,j,k)
	for(i=0;i<n-2;i++){//Keep Delaunay Triangle Coords
		for(j=i+1;j<n;j++){
			for (k=i+1;k<n;k++) {
				if (j!=k) {
					//Compute Normal to Triangle (i,j,k)
					xn = (y[j]-y[i])*(z[k]-z[i]) - (y[k]-y[i])*(z[j]-z[i]);
					yn = (x[k]-x[i])*(z[j]-z[i]) - (x[j]-x[i])*(z[k]-z[i]);
					zn = (x[j]-x[i])*(y[k]-y[i]) - (x[k]-x[i])*(y[j]-y[i]);

					flag = (zn < 0);	//Only examine faces on bottom of paraboloid: zn < 0
					if (flag){
						//For each other point m Check if m above (i,j,k)
						for (m=0;m<n; m++){
							flag = flag && ((x[m]-x[i])*xn + (y[m]-y[i])*yn + (z[m]-z[i])*zn <= 0);
						}
					}
					
					if (flag) {	
					//printf("[%g %g] [%g %g] [%g %g] [%g %g]\n",x[i], y[i], x[j], y[j], x[k], y[k], x[i], y[i]);
						DelaunayTriangle.i=i;
						DelaunayTriangle.j=j;
						DelaunayTriangle.k=k;

						Triangles.push_back(DelaunayTriangle);	//Keep Delaunay Triangle
					}
				}
			}
		}
	}


	//Alloc MeanDelaunayTriangles Memory
	CvMat*MeanDelaunayTriangles=cvCreateMat(Triangles.size(),3,CV_16UC1);

	//Copy Delaunay Triangulation Result 
	for(i=0;i<(int)Triangles.size();i++){
		cvM16i(MeanDelaunayTriangles,i,0)=Triangles[i].i;
		cvM16i(MeanDelaunayTriangles,i,1)=Triangles[i].j;
		cvM16i(MeanDelaunayTriangles,i,2)=Triangles[i].k;
	}


	//Memory Dellocation
	free(x);
	free(y);
	free(z);
}


 
